<?php
/**
 * A Magento 2 module named Experius/DonationProduct
 * Copyright (C) 2017 Derrick Heesbeen
 *
 * This file is part of Experius/DonationProduct.
 *
 * Experius/DonationProduct is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

namespace Experius\DonationProduct\Model\Product\Type;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Experius\DonationProduct\Helper\Data;

/**
 * Class Donation
 * @package Experius\DonationProduct\Model\Product\Type
 */
class Donation extends \Magento\Catalog\Model\Product\Type\AbstractType
{

    const TYPE_CODE = 'donation';

    /**
     * @var Data
     */
    protected $donationProductHelper;

    /**
     * Donation constructor.
     * @param \Magento\Catalog\Model\Product\Option $catalogProductOption
     * @param \Magento\Eav\Model\Config $eavConfig
     * @param \Magento\Catalog\Model\Product\Type $catalogProductType
     * @param \Magento\Framework\Event\ManagerInterface $eventManager
     * @param \Magento\MediaStorage\Helper\File\Storage\Database $fileStorageDb
     * @param \Magento\Framework\Filesystem $filesystem
     * @param \Magento\Framework\Registry $coreRegistry
     * @param \Psr\Log\LoggerInterface $logger
     * @param ProductRepositoryInterface $productRepository
     * @param Data $donationProductHelper
     */
    public function __construct(
        \Magento\Catalog\Model\Product\Option $catalogProductOption,
        \Magento\Eav\Model\Config $eavConfig,
        \Magento\Catalog\Model\Product\Type $catalogProductType,
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \Magento\MediaStorage\Helper\File\Storage\Database $fileStorageDb,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\Registry $coreRegistry,
        \Psr\Log\LoggerInterface $logger,
        ProductRepositoryInterface $productRepository,
        Data $donationProductHelper
    ) {
        $this->_catalogProductOption = $catalogProductOption;
        $this->_eavConfig = $eavConfig;
        $this->_catalogProductType = $catalogProductType;
        $this->_coreRegistry = $coreRegistry;
        $this->_eventManager = $eventManager;
        $this->_fileStorageDb = $fileStorageDb;
        $this->_filesystem = $filesystem;
        $this->_logger = $logger;
        $this->productRepository = $productRepository;
        $this->donationProductHelper = $donationProductHelper;

        parent::__construct(
            $catalogProductOption,
            $eavConfig,
            $catalogProductType,
            $eventManager,
            $fileStorageDb,
            $filesystem,
            $coreRegistry,
            $logger,
            $productRepository
        );
    }

    /**
     * @param \Magento\Catalog\Model\Product $product
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function checkProductBuyState($product)
    {
        parent::checkProductBuyState($product);

        if (!$product->getCustomOption(Data::DONATION_OPTION_CODE)) {
            throw new \Magento\Framework\Exception\LocalizedException($this->getSpecifyOptionMessage());
        }

        return $this;
    }

    /**
     * @param \Magento\Framework\DataObject $buyRequest
     * @param \Magento\Catalog\Model\Product $product
     * @param string $processMode
     * @return array|string
     */
    public function _prepareProduct(\Magento\Framework\DataObject $buyRequest, $product, $processMode)
    {
        if (!$product->getCustomOption(Data::DONATION_OPTION_CODE)) {
            return $this->getSpecifyOptionMessage()->render();
        }

        return parent::_prepareProduct($buyRequest, $product, $processMode); // TODO: Change the autogenerated stub
    }

    /**
     * @return \Magento\Framework\Phrase
     */
    public function getSpecifyOptionMessage()
    {
        return __('You need to choose a amount to donate');
    }

    /**
     * @param \Magento\Catalog\Model\Product $product
     * @return array
     */
    public function getOrderOptions($product)
    {
        $options = parent::getOrderOptions($product);

        if ($configurationOption = $product->getCustomOption(Data::DONATION_OPTION_CODE)) {
            $optionsArray = $this->donationProductHelper->optionsJsonToMagentoOptionsArray(
                $configurationOption->getValue(),
                $product
            );
            $currentOptions = (isset($options['options'])) ? $options['options'] : [];
            $options['options'] = array_merge($currentOptions, $optionsArray);
        }

        return $options;
    }

    /**
     * @param \Magento\Catalog\Model\Product $product
     * @return bool
     */
    public function isVirtual($product)
    {
        return true;
    }

    /**
     * @return bool
     */
    public function hasWeight()
    {
        return false;
    }

    /**
     * @param \Magento\Catalog\Model\Product $product
     */
    public function deleteTypeSpecificData(\Magento\Catalog\Model\Product $product)
    {
    }

    /**
     * @param \Magento\Framework\DataObject $buyRequest
     * @param \Magento\Catalog\Model\Product $product
     * @param null $processMode
     * @return array|string
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function prepareForCartAdvanced(\Magento\Framework\DataObject $buyRequest, $product, $processMode = null)
    {

        $typeId = $product->getTypeId();

        if ($typeId == self::TYPE_CODE && $buyRequest->getData('amount') ||
            $typeId == self::TYPE_CODE && $buyRequest->getData('amount_fixed')
        ) {
            $amountFixed = $buyRequest->getData('amount_fixed');
            $amount = $buyRequest->getData('amount');
            $finalAmount = ($amount>0) ? $amount : $amountFixed;

            if (!$finalAmount) {
                throw new \Magento\Framework\Exception\LocalizedException(__('Please enter a donation amount'));
            }

            if ($finalAmount<$this->donationProductHelper->getMinimalAmount($product)) {
                throw new \Magento\Framework\Exception\LocalizedException(
                    __('Donation amount lower then minimal amount')
                );
            }

            if ($finalAmount>$this->donationProductHelper->getMaximalAmount($product)) {
                throw new \Magento\Framework\Exception\LocalizedException(
                    __('Donation amount higher then maximal amount')
                );
            }

            $donationData['amount'] = $finalAmount;
            $product->addCustomOption(Data::DONATION_OPTION_CODE, json_encode($donationData));
        }

        return parent::prepareForCartAdvanced($buyRequest, $product, $processMode);
    }
}
